/**
 * Copyright (C) 2007-2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>

#include <glib/gstdio.h>

#include "utils.h"
#include "wk-key-file.h"
#include "wk-settings.h"

void
wk_settings_set_defaults (WkSettings **settings, WkKeyFile *kf, const gchar *uri)
{
    WkSettings *lsettings;

    lsettings = g_new (WkSettings, 1);

    lsettings->hook_dir = g_strdup (wk_key_file_get_filename (
                                kf, WK_KEY_FILE_FILENAME_HOOKS));
    lsettings->list_file = g_strdup (wk_key_file_get_filename (
                                kf, WK_KEY_FILE_FILENAME_LIST));
    lsettings->prestart_file = g_strdup (wk_key_file_get_filename (
                                kf, WK_KEY_FILE_FILENAME_PRESTART));
    lsettings->playlist_file = g_strdup (wk_key_file_get_filename (
                                kf, WK_KEY_FILE_FILENAME_PLAYLIST));
    lsettings->cookies_file = g_strdup (wk_key_file_get_filename (
                                kf, WK_KEY_FILE_FILENAME_COOKIES));

    lsettings->mode = MODE_KIOSK;
    lsettings->can_end_session = PREF_CAN_END_SESSION;
    lsettings->can_access_fs = PREF_CAN_ACCESS_FS;
    lsettings->fullscreen = PREF_FULLSCREEN;
    lsettings->with_timer = PREF_WITH_TIMER;
    lsettings->default_timer = PREF_DEFAULT_TIMER;
    lsettings->with_clear = PREF_WITH_CLEAR;
    lsettings->default_url = g_strdup (PREF_DEFAULT_URL);
    lsettings->after_web_session = PREF_DEFAULT_AFTER_WEB_SESSION;
    lsettings->max_history = PREF_MAX_HISTORY;
    lsettings->max_windows = PREF_MAX_WINDOWS;
    lsettings->user_agent = NULL;

    lsettings->list_mode = LIST_MODE_NONE;
    lsettings->list = NULL;

    lsettings->prestart = PREF_PRESTART;
    lsettings->start_with_session = PREF_START_WITH_SESSION;
    lsettings->hide_toolbar = PREF_HIDE_TOOLBAR;
    lsettings->stop_session_after_seconds = PREF_STOP_SESSION_AFTER_SECONDS;

    lsettings->post_session_uri = NULL;
    lsettings->post_session_uri_time = PREF_POST_SESSION_URI_TIME;

    lsettings->slideshow_hide_pointer = PREF_SLIDESHOW_HIDE_POINTER;
    lsettings->slideshow_mute = PREF_SLIDESHOW_MUTE;
    lsettings->slideshow_stop_on_event = PREF_SLIDESHOW_STOP_ON_EVENT;
    lsettings->slideshow_refresh_after_minutes = PREF_SLIDESHOW_REFRESH;

    lsettings->mime_associations = NULL;

    lsettings->admin_passwd = NULL;

    lsettings->tmpdir_name = g_strdup_printf ("webkiosk-%d", getuid ());

    /* we assume that settings is not allocated yet */
    *settings = lsettings;
}

void
wk_settings_get (WkSettings **settings, WkKeyFile *kf, const gchar *uri)
{
    WkSettings *lsettings = NULL;
    gchar *tmp = NULL;
    gchar **types = NULL;
    gboolean list_use_regex;
    gchar *contents = NULL;

    lsettings = g_new (WkSettings, 1);

    tmp = wk_key_file_get_string (kf, "main", "mode", "kiosk");
    if (strcmp (tmp, "kiosk") == 0)
        lsettings->mode = MODE_KIOSK;
    else
        lsettings->mode = MODE_DISPLAY;
    g_free (tmp);

    /* set a few things */
    lsettings->list = NULL;
    lsettings->post_session_uri = NULL;
    lsettings->mime_associations = NULL;

    lsettings->hook_dir = g_strdup (wk_key_file_get_filename (
                                kf, WK_KEY_FILE_FILENAME_HOOKS));
    lsettings->list_file = g_strdup (wk_key_file_get_filename (
                                kf, WK_KEY_FILE_FILENAME_LIST));
    lsettings->playlist_file = g_strdup (wk_key_file_get_filename (
                                kf, WK_KEY_FILE_FILENAME_PLAYLIST));
    lsettings->prestart_file = g_strdup (wk_key_file_get_filename (
                                kf, WK_KEY_FILE_FILENAME_PRESTART));
    lsettings->cookies_file = g_strdup (wk_key_file_get_filename (
                                kf, WK_KEY_FILE_FILENAME_COOKIES));

    lsettings->can_end_session = wk_key_file_get_boolean (
        kf, "main", "can_end_session", PREF_CAN_END_SESSION);
    lsettings->can_access_fs = wk_key_file_get_boolean (
        kf, "main", "can_access_fs", PREF_CAN_ACCESS_FS);
    lsettings->fullscreen = wk_key_file_get_boolean (
        kf, "main", "fullscreen", PREF_FULLSCREEN);
    lsettings->default_timer = wk_key_file_get_int (
        kf, "main", "default_timer", PREF_DEFAULT_TIMER);
    lsettings->with_timer = wk_key_file_get_boolean (
        kf, "main", "with_timer", PREF_WITH_TIMER);
    lsettings->with_clear = wk_key_file_get_boolean (
        kf, "main", "with_clear", PREF_WITH_CLEAR);
    lsettings->max_history = wk_key_file_get_int (
        kf, "main", "max_history", PREF_MAX_HISTORY);
    lsettings->max_windows = wk_key_file_get_int (
        kf, "main", "max_windows", PREF_MAX_WINDOWS);
    lsettings->hide_toolbar = wk_key_file_get_boolean (
        kf, "main", "hide_toolbar", PREF_HIDE_TOOLBAR);

    lsettings->default_url = wk_key_file_get_string (
        kf, "main", "default_url", PREF_DEFAULT_URL);

    lsettings->user_agent = wk_key_file_get_string (
        kf, "main", "user_agent", NULL);

    lsettings->post_session_uri_time = wk_key_file_get_int (
        kf, "main", "post_session_uri_time", PREF_POST_SESSION_URI_TIME);
    lsettings->post_session_uri = wk_key_file_get_string (
        kf, "main", "post_session_uri", NULL);

    lsettings->prestart = wk_key_file_get_boolean (
        kf, "main", "prestart", PREF_PRESTART);
    if (!lsettings->prestart)
    {
        lsettings->start_with_session = wk_key_file_get_boolean (
            kf, "main", "start_with_session", PREF_START_WITH_SESSION);
    }
    lsettings->stop_session_after_seconds = wk_key_file_get_int (
        kf, "main", "stop_session_after_seconds", PREF_STOP_SESSION_AFTER_SECONDS);

    tmp = wk_key_file_get_string (kf, "main", "after_web_session", NULL);
    if (g_strcmp0 (tmp, "slideshow") == 0)
        lsettings->after_web_session = AFTER_WEB_SESSION_SLIDESHOW;
    else if (g_strcmp0 (tmp, "session") == 0)
        lsettings->after_web_session = AFTER_WEB_SESSION_SESSION;
    else
        lsettings->after_web_session = AFTER_WEB_SESSION_HOMEPAGE;
    g_free (tmp);
    tmp = NULL;

    tmp = wk_key_file_get_string (kf, "main", "list_mode", NULL);
    if (g_strcmp0 (tmp, "whitelist") == 0)
        lsettings->list_mode = LIST_MODE_WHITELIST;
    else if (g_strcmp0 (tmp, "blacklist") == 0)
        lsettings->list_mode = LIST_MODE_BLACKLIST;
    else
        lsettings->list_mode = LIST_MODE_NONE;
    g_free (tmp);
    tmp = NULL;

    list_use_regex = wk_key_file_get_boolean (
        kf, "main", "list_use_regex", PREF_LIST_USE_REGEX);

    if (lsettings->list_mode != LIST_MODE_NONE)
    {
        gchar **lines = NULL;
        gchar **linep = NULL;

        if (g_file_get_contents (lsettings->list_file, &contents, NULL, NULL))
        {
            lines = g_strsplit (contents, "\n", -1);
            for (linep = lines; *linep; linep++)
            {
                if (strlen (*linep) == 0)
                    continue;

                gchar *regex;
                regex = list_use_regex ? g_strdup (*linep)
                                       : wildcard_to_regex (*linep);
                if (regex)
                    lsettings->list = g_list_append (lsettings->list, regex);

                /* the pointer to *regex is held by the list, we don't free it */
            }
            g_free (contents);
            contents = NULL;
            g_strfreev (lines);
        }
    }

    lsettings->slideshow_hide_pointer = wk_key_file_get_boolean (
        kf, "slideshow", "hide_pointer", PREF_SLIDESHOW_HIDE_POINTER);
    lsettings->slideshow_mute = wk_key_file_get_boolean (
        kf, "slideshow", "mute", PREF_SLIDESHOW_MUTE);
    lsettings->slideshow_stop_on_event = wk_key_file_get_boolean (
        kf, "slideshow", "stop_on_event", PREF_SLIDESHOW_STOP_ON_EVENT);
    lsettings->slideshow_refresh_after_minutes = wk_key_file_get_int (
        kf, "slideshow", "refresh_after_minutes", PREF_SLIDESHOW_REFRESH);
    tmp = wk_key_file_get_string (kf, "slideshow", "pages", "");
    if (strcmp (tmp, "") == 0)
        lsettings->slideshow_file = g_strdup (wk_key_file_get_filename (
                                       kf, WK_KEY_FILE_FILENAME_PLAYLIST));
    else
        lsettings->slideshow_file = g_strdup (tmp);
    g_free (tmp);
    tmp = NULL;

    types = wk_key_file_get_keys (kf, "mimetypes", NULL, NULL);
    if (G_LIKELY (types))
    {
        lsettings->mime_associations = g_hash_table_new (g_str_hash, g_str_equal);
        gchar **ptr;
        for (ptr = types; *ptr; ptr++)
        {
            /*
             * if no application is specified, we try to open the file through
             * xdg methods
             */
            g_hash_table_insert (lsettings->mime_associations,
                                 *ptr,
                                 wk_key_file_get_string (kf, "mimetypes",
                                                         *ptr, "xdg_open"));
        }
        /* free the array but not its content, held by the hash table */
        g_free (types);
        types = NULL;
    }

    lsettings->admin_passwd = wk_key_file_get_string (
        kf, "main", "admin_passwd", NULL);
    if (g_strcmp0 (lsettings->admin_passwd, "") == 0)
    {
        g_free (lsettings->admin_passwd);
        lsettings->admin_passwd = NULL;
    }

    lsettings->tmpdir_name = g_strdup_printf ("webkiosk-%d", getuid ());

    g_object_unref (kf);

    /* we assume that settings is not allocated yet */
    *settings = lsettings;
}

void
wk_settings_load_from_uri (WkSettings **settings, const gchar *uri)
{
    WkKeyFile *kf;

    kf = wk_key_file_new ((gchar *) uri);

    if (wk_key_file_load (kf))
        wk_settings_get (settings, kf, uri);
    else
        wk_settings_set_defaults (settings, kf, uri);
}

void
list_free_each (gpointer a, gpointer b)
{
    g_free ((gchar *) a);
}

void
wk_settings_free (WkSettings *settings)
{
    g_free (settings->hook_dir);
    g_free (settings->list_file);
    g_free (settings->playlist_file);
    g_free (settings->prestart_file);
    g_free (settings->cookies_file);
    g_free (settings->user_agent);

    g_free (settings->default_url);
    g_free (settings->tmpdir_name);

    g_list_foreach (settings->list, list_free_each, NULL);
    g_list_free (settings->list);

    if (settings->mime_associations)
        g_hash_table_destroy (settings->mime_associations);
}


